#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>

#define IN 0
#define OUT 1
#define LOW 0
#define HIGH 1
#define PIN 21
#define POUT 17
#define BUTTON 20

static int GPIOExport(int pin) {
	#define BUFFER_MAX 3
	char buffer[BUFFER_MAX];
	ssize_t bytes_written;
	int fd;
	
	fd = open("/sys/class/gpio/export", O_WRONLY);
	
	bytes_written = snprintf(buffer, BUFFER_MAX, "%d", pin);
	write(fd, buffer, bytes_written);
	close(fd);
	return 0;
}

static int GPIODirection(int pin, int dir) {
	static const char s_directions_str[] = "in\0out";
	#define DIRECTION_MAX 35
	
	char path[DIRECTION_MAX] = "/sys/class/gpio/gpio%d/direction";
	int fd;
	
	snprintf(path, DIRECTION_MAX, "/sys/class/gpio/gpio%d/direction", pin);
	
	fd = open(path, O_WRONLY);
	write(fd, &s_directions_str[IN == dir ? 0:3], IN == dir ? 2:3);
	
	close(fd);
	return 0;
}

static int GPIOWrite(int pin, int value) {
	#define VALUE_MAX 30
	static const char s_values_str[] = "01";
	
	char path[VALUE_MAX];
	int fd;
	
	snprintf(path, VALUE_MAX, "/sys/class/gpio/gpio%d/value", pin);
	
	fd = open(path, O_WRONLY);
	write(fd, &s_values_str[LOW == value ? 0:1], 1);
	
	close(fd);
	return 0;
}

static int GPIORead(int pin) {
	#define VALUE_MAX 30
	char value_str[3];
	char path[VALUE_MAX];
	int fd;
	
	snprintf(path, VALUE_MAX, "/sys/class/gpio/gpio%d/value", pin);
	
	fd = open(path, O_RDONLY);
	read(fd, (void*)value_str, 3);
	
	close(fd);
	return atoi(value_str);
}

static int GPIOUnexport(int pin){
	char buffer[BUFFER_MAX];
	ssize_t bytes_written;
	
	int fd;
	
	fd =open("/sys/class/gpio/unexport", O_WRONLY);
	
	bytes_written = snprintf(buffer, BUFFER_MAX	, "%d", pin);
	write(fd, buffer, bytes_written);
	close(fd);
	return 0;
}

void (*breakCapture)(int);

void signalingHandler(int signo){
	GPIOUnexport(POUT);
	GPIOUnexport(PIN);
	GPIOUnexport(BUTTON);
	printf("exit program correctly!!\n");
	exit(1);
}

int main(int argc, char *argv[]){
	int state = 0;
	int state_btn = 1;
	
	setsid();
	umask(0);
	
	breakCapture = signal(SIGINT, signalingHandler);
	
	if (-1 == GPIOExport(POUT) || GPIOExport(BUTTON) || GPIOExport(PIN))
		return 1;
		
	if (-1 == GPIODirection(POUT, OUT) || GPIODirection(BUTTON, IN) || GPIODirection(PIN, OUT))
	if (-1 == GPIODirection(POUT, OUT) || GPIODirection(BUTTON, IN) || GPIODirection(PIN, OUT))
		return 2;
	
	do {
		GPIOWrite(PIN, HIGH);
		while(GPIORead(BUTTON)==state_btn) continue;
		printf("button is pushed!!\n");
		//printf("Reading button %d\n", GPIORead(BUTTON));
		
		if (state_btn ^= 1)
			if (-1 == GPIOWrite(POUT, state ^= 1))
				return 3;
			
		//usleep(500 * 1000);
	} while (1);
	
	if (-1 == GPIOUnexport(POUT) || GPIOUnexport(BUTTON))
		return 4;
		
	return 0;
}



