#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <pthread.h>

#define BUFFER_MAX 3
#define DIRECTION_MAX 45

#define IN 0
#define OUT 1

#define LOW 0
#define HIGH 1
#define VALUE_MAX 256

#define PWM 0
#define POUT 23
#define PIN 24

double distance = 0;


static int PWMExport (int pwmnum){
	char buffer[BUFFER_MAX];
	int bytes_written;
	int fd;
	
	fd = open("/sys/class/pwm/pwmchip0/unexport", O_WRONLY);
	
	bytes_written = snprintf(buffer, BUFFER_MAX, "%d", pwmnum);
	write(fd, buffer, bytes_written);
	close(fd);
	
	sleep(1);
	fd = open("/sys/class/pwm/pwmchip0/export", O_WRONLY);
	bytes_written = snprintf(buffer, BUFFER_MAX, "%d", pwmnum);
	write(fd, buffer, bytes_written);
	close(fd);
	sleep(1);
	return 0;
}

static int PWMUnexport (int pwmnum){
	char buffer[BUFFER_MAX];
	int bytes_written;
	int fd;
	
	fd = open("/sys/class/pwm/pwmchip0/unexport", O_WRONLY);
	
	bytes_written = snprintf(buffer, BUFFER_MAX, "%d", pwmnum);
	write(fd, buffer, bytes_written);
	close(fd);
	
	sleep(1);
	
	return 0;
}

static int PWMEnable(int pwmnum){
	static const char s_unenable_str[] = "0";
	static const char s_enable_str[] = "1";
	
	char path[DIRECTION_MAX];
	int fd;
	
	snprintf(path, DIRECTION_MAX, "/sys/class/pwm/pwmchip0/pwm%d/enable", pwmnum);
	fd = open(path, O_WRONLY);
	
	write(fd, s_unenable_str, strlen(s_unenable_str));
	close(fd);
	
	fd=open(path, O_WRONLY);
	
	write(fd, s_enable_str, strlen(s_enable_str));
	close(fd);
	return(0);
}

static int PWMWritePeriod(int pwmnum, int value){
	char s_values_str[VALUE_MAX];
	char path[VALUE_MAX];
	int fd,byte;
	
	fd = open(path, O_WRONLY);
	snprintf(path, VALUE_MAX	, "/sys/class/pwm/pwmchip0/pwm%d/period", pwmnum);
	byte = snprintf(s_values_str, VALUE_MAX, "%d", value);
	write(fd, s_values_str, byte);
	
	close(fd);
	return(0);
}

static int PWMWriteDutyCycle(int pwmnum, int value){
	char path[VALUE_MAX];
	char s_values_str[VALUE_MAX];
	int fd,byte;
	
	snprintf(path, VALUE_MAX, "/sys/class/pwm/pwmchip0/pwm%d/duty_cycle", pwmnum);
	fd = open(path, O_WRONLY);
	
	byte= snprintf(s_values_str, VALUE_MAX, "%d", value);
	write(fd, s_values_str, byte);
	close(fd);
	
	return(0);
}


static int GPIOExport(int pin) {
	char buffer[BUFFER_MAX];
	ssize_t bytes_written;
	int fd;
	
	fd = open("/sys/class/gpio/export", O_WRONLY);
	
	bytes_written = snprintf(buffer, BUFFER_MAX, "%d", pin);
	write(fd, buffer, bytes_written);
	close(fd);
	return 0;
}

static int GPIOUnexport(int pin){
	char buffer[BUFFER_MAX];
	ssize_t bytes_written;
	
	int fd;
	
	fd =open("/sys/class/gpio/unexport", O_WRONLY);
	
	bytes_written = snprintf(buffer, BUFFER_MAX	, "%d", pin);
	write(fd, buffer, bytes_written);
	close(fd);
	return 0;
}

static int GPIODirection(int pin, int dir) {
	static const char s_directions_str[] = "in\0out";
	
	char path[DIRECTION_MAX] = "/sys/class/gpio/gpio%d/direction";
	int fd;
	
	snprintf(path, DIRECTION_MAX, "/sys/class/gpio/gpio%d/direction", pin);
	
	fd = open(path, O_WRONLY);
	write(fd, &s_directions_str[IN == dir ? 0:3], IN == dir ? 2:3);
	
	close(fd);
	return 0;
}

static int GPIOWrite(int pin, int value) {
	static const char s_values_str[] = "01";
	
	char path[VALUE_MAX];
	int fd;
	
	snprintf(path, VALUE_MAX, "/sys/class/gpio/gpio%d/value", pin);
	
	fd = open(path, O_WRONLY);
	write(fd, &s_values_str[LOW == value ? 0:1], 1);
	
	close(fd);
	return 0;
}

static int GPIORead(int pin) {
	char value_str[3];
	char path[VALUE_MAX];
	int fd;
	
	snprintf(path, VALUE_MAX, "/sys/class/gpio/gpio%d/value", pin);
	
	fd = open(path, O_RDONLY);
	read(fd, (void*)value_str, 3);
	
	close(fd);
	return atoi(value_str);
}

void *ultrawave_thd(){
	clock_t start_t, end_t;
	double time;
	
	if(-1 == GPIOExport(POUT) || -1 == GPIOExport(PIN)){
		printf("gpio export err\n");
		return 1;
	}
	
	usleep(100000);
	
	if(-1 == GPIODirection(POUT, OUT) || -1 == GPIODirection(PIN, IN)){
		printf("gpio direction err\n");
		return 1;
	}
	
	GPIOWrite(POUT, 0);
	usleep(10000);
	
	while (1){
		GPIOWrite(POUT, 1);
		
		usleep(10);
		GPIOWrite(POUT, 0);
		
		while(GPIORead(PIN) == 0){
			start_t = clock();
		}
		while(GPIORead(PIN) ==1) {
			end_t = clock();
		}
		
		time = (double)(end_t-start_t)/CLOCKS_PER_SEC;
		distance = time/2*34000;
		
		if(distance > 900)
			distance = 900;
		
		printf("time: %.4lf\n", time);
		printf("distance : %.2lfcm\n", distance);
		
		usleep(500000);
	}
	
	
	printf("complete\n");
	exit(0);
}

void *led_thd(){
	int target_bright=0;
	int prev_bright=0;
	
	PWMExport(0);
	PWMWritePeriod(0, 15000000);
	PWMWriteDutyCycle(0, 0);
	PWMEnable(0);
	
	while(1) {
		target_bright = (int)1/distance * 22000 * 900;
		printf("target_bright: %d\n", target_bright);
		if(target_bright > prev_bright){
			for(int i=prev_bright; i<target_bright; i+=4000){
				PWMWriteDutyCycle(0, i);
				usleep(1000);
			}
			
		} else {
			for(int i=prev_bright; i>target_bright; i-=4000){
				PWMWriteDutyCycle(0, i);
				usleep(1000);
			}
		}
		
		prev_bright=target_bright;
		usleep(1000);
	}
	
	exit(0);
}

int main(void){
	
	
	pthread_t p_thread[2];
	int thr_id;
	int status;
	char p1[] = "thread_1";
	char p2[] = "thread_2";
	
	thr_id = pthread_create(&p_thread[0], NULL, ultrawave_thd, NULL);
	thr_id = pthread_create(&p_thread[1], NULL, led_thd, NULL);
	
	
	pthread_join(p_thread[0], (void **)&status);
	pthread_join(p_thread[1], (void **)&status);
	
	GPIOUnexport(POUT);
	GPIOUnexport(PIN);
	PWMUnexport(0);
	return 0;
}
